@@ -17,6 +17,8 @@
 */
 
 #include "phar_internal.h"
+#include "ext/standard/crc32.h"
+#include "ext/standard/php_string.h" /* For php_stristr() */
 
 #define PHAR_GET_16(var) ((uint16_t)((((uint16_t)var[0]) & 0xff) | \
 	(((uint16_t)var[1]) & 0xff) << 8))
@@ -44,6 +46,7 @@ static int phar_zip_process_extra(php_stream *fp, phar_entry_info *entry, uint16
 	union {
 		phar_zip_extra_field_header header;
 		phar_zip_unix3 unix3;
+		phar_zip_unix_time time;
 	} h;
 	size_t read;
 
@@ -52,6 +55,35 @@ static int phar_zip_process_extra(php_stream *fp, phar_entry_info *entry, uint16
 			return FAILURE;
 		}
 
+		if (h.header.tag[0] == 'U' && h.header.tag[1] == 'T') {
+			/* Unix timestamp header found.
+			 * The flags field indicates which timestamp fields are present.
+			 * The size with a timestamp is at least 5 (== 9 - tag size) bytes, but may be larger.
+			 * We only store the modification time in the entry, so only read that.
+			 */
+			const size_t min_size = 5;
+			uint16_t header_size = PHAR_GET_16(h.header.size);
+			if (header_size >= min_size) {
+				read = php_stream_read(fp, &h.time.flags, min_size);
+				if (read != min_size) {
+					return FAILURE;
+				}
+				if (h.time.flags & (1 << 0)) {
+					/* Modification time set */
+					entry->timestamp = PHAR_GET_32(h.time.time);
+				}
+
+				len -= header_size + 4;
+
+				/* Consume remaining bytes */
+				if (header_size != read) {
+					php_stream_seek(fp, header_size - read, SEEK_CUR);
+				}
+				continue;
+			}
+			/* Fallthrough to next if to skip header */
+		}
+
 		if (h.header.tag[0] != 'n' || h.header.tag[1] != 'u') {
 			/* skip to next header */
 			php_stream_seek(fp, PHAR_GET_16(h.header.size), SEEK_CUR);
@@ -120,7 +152,7 @@ static int phar_zip_process_extra(php_stream *fp, phar_entry_info *entry, uint16
   OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN
   IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
-static time_t phar_zip_d2u_time(char *cdtime, char *cddate) /* {{{ */
+static time_t phar_zip_d2u_time(const char *cdtime, const char *cddate) /* {{{ */
 {
 	int dtime = PHAR_GET_16(cdtime), ddate = PHAR_GET_16(cddate);
 	struct tm *tm, tmbuf;
@@ -313,8 +345,29 @@ foundit:
 	entry.is_zip = 1;
 	entry.fp_type = PHAR_FP;
 	entry.is_persistent = mydata->is_persistent;
+#define PHAR_ZIP_FAIL_FREE(errmsg, save) \
+			zend_hash_destroy(&mydata->manifest); \
+			HT_INVALIDATE(&mydata->manifest); \
+			zend_hash_destroy(&mydata->mounted_dirs); \
+			HT_INVALIDATE(&mydata->mounted_dirs); \
+			zend_hash_destroy(&mydata->virtual_dirs); \
+			HT_INVALIDATE(&mydata->virtual_dirs); \
+			php_stream_close(fp); \
+			phar_metadata_tracker_free(&mydata->metadata_tracker, mydata->is_persistent); \
+			if (mydata->signature) { \
+				efree(mydata->signature); \
+			} \
+			if (error) { \
+				spprintf(error, 4096, "phar error: %s in zip-based phar \"%s\"", errmsg, mydata->fname); \
+			} \
+			pefree(mydata->fname, mydata->is_persistent); \
+			if (mydata->alias) { \
+				pefree(mydata->alias, mydata->is_persistent); \
+			} \
+			pefree(mydata, mydata->is_persistent); \
+			efree(save); \
+			return FAILURE;
 #define PHAR_ZIP_FAIL(errmsg) \
-			efree(actual_alias); \
 			zend_hash_destroy(&mydata->manifest); \
 			HT_INVALIDATE(&mydata->manifest); \
 			zend_hash_destroy(&mydata->mounted_dirs); \
@@ -468,13 +521,14 @@ foundit:
 			mydata->sig_flags = PHAR_GET_32(sig);
 			if (FAILURE == phar_verify_signature(sigfile, php_stream_tell(sigfile), mydata->sig_flags, sig + 8, entry.uncompressed_filesize - 8, fname, &mydata->signature, &sig_len, error)) {
 				efree(sig);
-				php_stream_close(sigfile);
 				if (error) {
-					char errmsg[128];
-					snprintf(errmsg, sizeof(errmsg), "signature cannot be verified: %s", *error);
+					char *save;
+					php_stream_close(sigfile);
+					spprintf(&save, 4096, "signature cannot be verified: %s", *error);
 					efree(*error);
-					PHAR_ZIP_FAIL(errmsg);
+					PHAR_ZIP_FAIL_FREE(save, save);
 				} else {
+					php_stream_close(sigfile);
 					PHAR_ZIP_FAIL("signature cannot be verified");
 				}
 			}
@@ -619,7 +673,7 @@ foundit:
 					}
 				}
 
-				if (!entry.uncompressed_filesize) {
+				if (!entry.uncompressed_filesize || !actual_alias) {
 					php_stream_filter_remove(filter, 1);
 					pefree(entry.filename, entry.is_persistent);
 					PHAR_ZIP_FAIL("unable to read in alias, truncated");
@@ -652,7 +706,7 @@ foundit:
 					}
 				}
 
-				if (!entry.uncompressed_filesize) {
+				if (!entry.uncompressed_filesize || !actual_alias) {
 					php_stream_filter_remove(filter, 1);
 					pefree(entry.filename, entry.is_persistent);
 					PHAR_ZIP_FAIL("unable to read in alias, truncated");
@@ -675,7 +729,7 @@ foundit:
 					}
 				}
 
-				if (!entry.uncompressed_filesize) {
+				if (!entry.uncompressed_filesize || !actual_alias) {
 					pefree(entry.filename, entry.is_persistent);
 					PHAR_ZIP_FAIL("unable to read in alias, truncated");
 				}
@@ -792,7 +846,6 @@ int phar_open_or_create_zip(char *fname, size_t fname_len, char *alias, size_t a
 	}
 
 	if (phar->is_brandnew) {
-		phar->internal_file_start = 0;
 		phar->is_zip = 1;
 		phar->is_tar = 0;
 		return SUCCESS;
@@ -811,9 +864,9 @@ struct _phar_zip_pass {
 	php_stream *filefp;
 	php_stream *centralfp;
 	php_stream *old;
-	int free_fp;
-	int free_ufp;
 	char **error;
+	bool free_fp;
+	bool free_ufp;
 };
 /* perform final modification of zip contents for each file in the manifest before saving */
 static int phar_zip_changed_apply_int(phar_entry_info *entry, void *arg) /* {{{ */
@@ -1205,14 +1258,13 @@ static int phar_zip_applysignature(phar_archive_data *phar, struct _phar_zip_pas
 }
 /* }}} */
 
-int phar_zip_flush(phar_archive_data *phar, char *user_stub, zend_long len, int defaultstub, char **error) /* {{{ */
+void phar_zip_flush(phar_archive_data *phar, zend_string *user_stub, bool is_default_stub, char **error) /* {{{ */
 {
-	char *pos;
 	static const char newstub[] = "<?php // zip-based phar archive stub file\n__HALT_COMPILER();";
-	char halt_stub[] = "__HALT_COMPILER();";
+	static const char halt_stub[] = "__HALT_COMPILER();";
 
-	php_stream *stubfile, *oldfile;
-	int free_user_stub, closeoldfile = 0;
+	php_stream *oldfile;
+	bool must_close_old_file = false;
 	phar_entry_info entry = {0};
 	char *temperr = NULL;
 	struct _phar_zip_pass pass;
@@ -1223,7 +1275,7 @@ int phar_zip_flush(phar_archive_data *phar, char *user_stub, zend_long len, int
 	entry.flags = PHAR_ENT_PERM_DEF_FILE;
 	entry.timestamp = time(NULL);
 	entry.is_modified = 1;
-	entry.is_zip = 1;
+	entry.is_zip = true;
 	entry.phar = phar;
 	entry.fp_type = PHAR_MOD;
 
@@ -1231,7 +1283,7 @@ int phar_zip_flush(phar_archive_data *phar, char *user_stub, zend_long len, int
 		if (error) {
 			spprintf(error, 0, "internal error: attempt to flush cached zip-based phar \"%s\"", phar->fname);
 		}
-		return EOF;
+		return;
 	}
 
 	if (phar->is_data) {
@@ -1243,14 +1295,13 @@ int phar_zip_flush(phar_archive_data *phar, char *user_stub, zend_long len, int
 		entry.fp = php_stream_fopen_tmpfile();
 		if (entry.fp == NULL) {
 			spprintf(error, 0, "phar error: unable to create temporary file");
-			return EOF;
+			return;
 		}
 		if (phar->alias_len != php_stream_write(entry.fp, phar->alias, phar->alias_len)) {
-			php_stream_close(entry.fp);
 			if (error) {
 				spprintf(error, 0, "unable to set alias in zip-based phar \"%s\"", phar->fname);
 			}
-			return EOF;
+			return;
 		}
 
 		entry.uncompressed_filesize = entry.compressed_filesize = phar->alias_len;
@@ -1265,112 +1316,67 @@ int phar_zip_flush(phar_archive_data *phar, char *user_stub, zend_long len, int
 	/* register alias */
 	if (phar->alias_len) {
 		if (FAILURE == phar_get_archive(&phar, phar->fname, phar->fname_len, phar->alias, phar->alias_len, error)) {
-			return EOF;
+			return;
 		}
 	}
 
 	/* set stub */
-	if (user_stub && !defaultstub) {
-		if (len < 0) {
-			/* resource passed in */
-			if (!(php_stream_from_zval_no_verify(stubfile, (zval *)user_stub))) {
-				if (error) {
-					spprintf(error, 0, "unable to access resource to copy stub to new zip-based phar \"%s\"", phar->fname);
-				}
-				return EOF;
-			}
+	if (user_stub && !is_default_stub) {
+		char *pos = php_stristr(ZSTR_VAL(user_stub), halt_stub, ZSTR_LEN(user_stub), strlen(halt_stub));
 
-			if (len == -1) {
-				len = PHP_STREAM_COPY_ALL;
-			} else {
-				len = -len;
-			}
-
-			user_stub = 0;
-
-			// TODO: refactor to avoid reallocation ???
-//???		len = php_stream_copy_to_mem(stubfile, &user_stub, len, 0)
-			{
-				zend_string *str = php_stream_copy_to_mem(stubfile, len, 0);
-				if (str) {
-					len = ZSTR_LEN(str);
-					user_stub = estrndup(ZSTR_VAL(str), ZSTR_LEN(str));
-					zend_string_release_ex(str, 0);
-				} else {
-					user_stub = NULL;
-					len = 0;
-				}
-			}
-
-			if (!len || !user_stub) {
-				if (error) {
-					spprintf(error, 0, "unable to read resource to copy stub to new zip-based phar \"%s\"", phar->fname);
-				}
-				return EOF;
-			}
-			free_user_stub = 1;
-		} else {
-			free_user_stub = 0;
-		}
-
-		if ((pos = php_stristr(user_stub, halt_stub, len, sizeof(halt_stub) - 1)) == NULL) {
+		if (pos == NULL) {
 			if (error) {
 				spprintf(error, 0, "illegal stub for zip-based phar \"%s\"", phar->fname);
 			}
-			if (free_user_stub) {
-				efree(user_stub);
-			}
-			return EOF;
+			return;
 		}
 
-		len = pos - user_stub + 18;
+		size_t len = pos - ZSTR_VAL(user_stub) + strlen(halt_stub);
+		const char end_sequence[] = " ?>\r\n";
+		size_t end_sequence_len = strlen(end_sequence);
+
 		entry.fp = php_stream_fopen_tmpfile();
 		if (entry.fp == NULL) {
 			spprintf(error, 0, "phar error: unable to create temporary file");
-			return EOF;
+			return;
 		}
-		entry.uncompressed_filesize = len + 5;
+		entry.uncompressed_filesize = len + end_sequence_len;
 
-		if ((size_t)len != php_stream_write(entry.fp, user_stub, len)
-		||            5 != php_stream_write(entry.fp, " ?>\r\n", 5)) {
+		if (
+			len != php_stream_write(entry.fp, ZSTR_VAL(user_stub), len)
+			|| end_sequence_len != php_stream_write(entry.fp, end_sequence, end_sequence_len)
+		) {
 			if (error) {
 				spprintf(error, 0, "unable to create stub from string in new zip-based phar \"%s\"", phar->fname);
 			}
-			if (free_user_stub) {
-				efree(user_stub);
-			}
 			php_stream_close(entry.fp);
-			return EOF;
+			return;
 		}
 
 		entry.filename = estrndup(".phar/stub.php", sizeof(".phar/stub.php")-1);
 		entry.filename_len = sizeof(".phar/stub.php")-1;
 
 		zend_hash_str_update_mem(&phar->manifest, entry.filename, entry.filename_len, (void*)&entry, sizeof(phar_entry_info));
-
-		if (free_user_stub) {
-			efree(user_stub);
-		}
 	} else {
 		/* Either this is a brand new phar (add the stub), or the default stub is required (overwrite the stub) */
 		entry.fp = php_stream_fopen_tmpfile();
 		if (entry.fp == NULL) {
 			spprintf(error, 0, "phar error: unable to create temporary file");
-			return EOF;
+			return;
 		}
 		if (sizeof(newstub)-1 != php_stream_write(entry.fp, newstub, sizeof(newstub)-1)) {
 			php_stream_close(entry.fp);
 			if (error) {
 				spprintf(error, 0, "unable to %s stub in%szip-based phar \"%s\", failed", user_stub ? "overwrite" : "create", user_stub ? " " : " new ", phar->fname);
 			}
-			return EOF;
+			return;
 		}
 
 		entry.uncompressed_filesize = entry.compressed_filesize = sizeof(newstub) - 1;
 		entry.filename = estrndup(".phar/stub.php", sizeof(".phar/stub.php")-1);
 		entry.filename_len = sizeof(".phar/stub.php")-1;
 
-		if (!defaultstub) {
+		if (!is_default_stub) {
 			if (!zend_hash_str_exists(&phar->manifest, ".phar/stub.php", sizeof(".phar/stub.php")-1)) {
 				if (NULL == zend_hash_str_add_mem(&phar->manifest, entry.filename, entry.filename_len, (void*)&entry, sizeof(phar_entry_info))) {
 					php_stream_close(entry.fp);
@@ -1378,7 +1384,7 @@ int phar_zip_flush(phar_archive_data *phar, char *user_stub, zend_long len, int
 					if (error) {
 						spprintf(error, 0, "unable to create stub in zip-based phar \"%s\"", phar->fname);
 					}
-					return EOF;
+					return;
 				}
 			} else {
 				php_stream_close(entry.fp);
@@ -1391,11 +1397,11 @@ int phar_zip_flush(phar_archive_data *phar, char *user_stub, zend_long len, int
 nostub:
 	if (phar->fp && !phar->is_brandnew) {
 		oldfile = phar->fp;
-		closeoldfile = 0;
+		must_close_old_file = false;
 		php_stream_rewind(oldfile);
 	} else {
 		oldfile = php_stream_open_wrapper(phar->fname, "rb", 0, NULL);
-		closeoldfile = oldfile != NULL;
+		must_close_old_file = oldfile != NULL;
 	}
 
 	/* save modified files to the zip */
@@ -1404,19 +1410,18 @@ nostub:
 
 	if (!pass.filefp) {
 fperror:
-		if (closeoldfile) {
+		if (must_close_old_file) {
 			php_stream_close(oldfile);
 		}
 		if (error) {
 			spprintf(error, 4096, "phar zip flush of \"%s\" failed: unable to open temporary file", phar->fname);
 		}
-		return EOF;
+		return;
 	}
 
 	pass.centralfp = php_stream_fopen_tmpfile();
 
 	if (!pass.centralfp) {
-		php_stream_close(pass.filefp);
 		goto fperror;
 	}
 
@@ -1447,10 +1452,10 @@ notemperror:
 		php_stream_close(pass.centralfp);
 nocentralerror:
 		php_stream_close(pass.filefp);
-		if (closeoldfile) {
+		if (must_close_old_file) {
 			php_stream_close(oldfile);
 		}
-		return EOF;
+		return;
 	}
 
 	if (FAILURE == phar_zip_applysignature(phar, &pass)) {
@@ -1524,14 +1529,14 @@ nocentralerror:
 	} else {
 		phar->fp = php_stream_open_wrapper(phar->fname, "w+b", IGNORE_URL|STREAM_MUST_SEEK|REPORT_ERRORS, NULL);
 		if (!phar->fp) {
-			if (closeoldfile) {
+			if (must_close_old_file) {
 				php_stream_close(oldfile);
 			}
 			phar->fp = pass.filefp;
 			if (error) {
 				spprintf(error, 4096, "unable to open new phar \"%s\" for writing", phar->fname);
 			}
-			return EOF;
+			return;
 		}
 		php_stream_rewind(pass.filefp);
 		php_stream_copy_to_stream_ex(pass.filefp, phar->fp, PHP_STREAM_COPY_ALL, NULL);
@@ -1539,9 +1544,8 @@ nocentralerror:
 		php_stream_close(pass.filefp);
 	}
 
-	if (closeoldfile) {
+	if (must_close_old_file) {
 		php_stream_close(oldfile);
 	}
-	return 0;
 }
 /* }}} */
